PROGRAM Fuenferspiel;
  {nimmt rekursiven Graphdurchlauf vor}

  USES
    Crt;

  CONST
    zeilenzahl  =  2;
    spaltenzahl =  3;
    leer        = '#'; {symbolisiert leeres Feld}
    maxlaenge   =  80; {maximale Lnge der Besuchtliste}

  TYPE
    tFeld      = RECORD
                   zeile  : 0..zeilenzahl;
                   spalte : 0..spaltenzahl
                 END;
    tBrett     = ARRAY[1..zeilenzahl, 1..spaltenzahl] OF char;
    tPosition  = RECORD
                   brett         : tBrett;
                   leerZ, leerZ0 : tFeld;
                 END;
    tZug       = (west, nord, ost, sued);
    tListe     = RECORD
                   inhalt : ARRAY[1..maxlaenge] OF tPosition;
                   laenge : 0..maxlaenge
                 END;
  VAR
    anfangsposition  : tPosition;
    besuchtliste     : tListe;
    tiefenschranke   : integer;

  PROCEDURE Startangaben (VAR start: tPosition);
    {liest Startposition ein und prft Eingabe}
    TYPE
      tMenge = SET OF char;
    CONST
      zeichenmenge = ['1'..'5', leer];
    VAR
      korrekt : boolean;

    PROCEDURE Eingabe (VAR pos: tPosition; text: string);
      {liest Spielposition ein}
      VAR
        i, j    : integer;
        zeichen : char;
      BEGIN
        writeln('Geben Sie die ', text, ' ein!');
        WITH pos DO BEGIN
          FOR i := 1 TO zeilenzahl DO BEGIN
            writeln;
            FOR j := 1 TO spaltenzahl DO BEGIN
              write(i, '. Zeile und ', j, '. Spalte: ');
              REPEAT zeichen := readKey UNTIL zeichen IN zeichenmenge;
              writeln(zeichen); brett[i,j] := zeichen;
              IF zeichen = leer THEN BEGIN
                leerZ.zeile  := i; leerZ.spalte  := j
              END {of then}
            END {of for}
          END; {of for}
          leerZ0.zeile := 0; leerZ0.spalte := 0
        END {of with}
      END; {of Eingabe}

    FUNCTION inOrdnung (pos: tPosition): boolean;
      {prft, ob Position in Ordnung ist}
      VAR
        i, j          : integer;
        kontrollmenge : tMenge;
      BEGIN
        kontrollmenge := [];
        FOR i := 1 TO zeilenzahl DO
          FOR j := 1 TO spaltenzahl DO
            kontrollmenge := kontrollmenge + [pos.brett[i,j]];
        inOrdnung := kontrollmenge = zeichenmenge
      END; {of in Ordnung}

    BEGIN {Startangaben}
      clrScr;
      REPEAT
        Eingabe(start, 'Anfangsposition');
        korrekt := inOrdnung(start);
        IF NOT korrekt THEN BEGIN
          writeln; writeln('Eingabe fehlerhaft!')
        END
      UNTIL korrekt;
      REPEAT
        writeln; write('Tiefenschranke (<= ', maxlaenge, '): ');
        readln(tiefenschranke)
      UNTIL tiefenschranke <= maxlaenge;
      besuchtliste.laenge := 0
    END; {of Startangaben}

  PROCEDURE Ausgabe (pos: tPosition; x, y: integer);
    {gibt einzelne Spielposition in Spalte x, Zeile y aus}
    VAR
      i, j : integer;
    BEGIN
      WITH pos DO
        FOR i := 1 TO zeilenzahl DO
          FOR j := 1 TO spaltenzahl DO BEGIN
            gotoXY(x + 2*(j-1), y + i-1); write(brett[i,j])
          END {of for}
    END; {of Ausgabe}

  FUNCTION zulaessig (zug: tZug; pos: tPosition): boolean;
    {prft Zug auf Zulssigkeit}
    BEGIN
      WITH pos DO
        CASE zug OF
          west : zulaessig := (leerZ.spalte > 1) AND
                              (leerZ.spalte - 1 <> leerZ0.spalte);
          nord : zulaessig := (leerZ.zeile  > 1) AND
                              (leerZ.zeile - 1 <> leerZ0.zeile);
          ost  : zulaessig := (leerZ.spalte < spaltenzahl) AND
                              (leerZ.spalte + 1 <> leerZ0.spalte);
          sued : zulaessig := (leerZ.zeile  < zeilenzahl) AND
                              (leerZ.zeile + 1 <> leerZ0.zeile)
        END {of case}
    END; {of zulssig}

  PROCEDURE erzeuge (VAR neu: tPosition; zug: tZug; alt: tPosition);
    {erzeugt aus alter Position und Zug eine neue Position}
    BEGIN
      neu := alt; neu.leerZ0 := neu.leerZ;
      CASE zug OF
        west : dec(neu.leerZ.spalte);
        nord : dec(neu.leerZ.zeile);
        ost  : inc(neu.leerZ.spalte);
        sued : inc(neu.leerZ.zeile)
      END; {of case}
      neu.brett[neu.leerZ.zeile, neu.leerZ.spalte] := leer;
      neu.brett[alt.leerZ.zeile, alt.leerZ.spalte] :=
                            alt.brett[neu.leerZ.zeile, neu.leerZ.spalte]
    END; {of erzeuge}

  PROCEDURE fuege_an (pos: tPosition; i: integer; VAR liste: tListe);
    {fgt Spielposition an Liste an}
    BEGIN
      WITH liste DO
        IF i >= maxlaenge THEN BEGIN
          clrScr; gotoXY(25, 10);
          writeln('Die Liste ist voll!'); readln; halt
        END ELSE BEGIN
          inhalt[i] := pos; laenge := i
        END {of else}
    END; {of fge an}

  FUNCTION gleich (pos1, pos2: tPosition): boolean;
    {prft, ob zwei Positionen gleich sind}
    VAR
      i, j : integer;
    BEGIN
      gleich := false;
      FOR i := 1 TO zeilenzahl DO
        FOR j := 1 TO spaltenzahl DO
          IF pos1.brett[i,j] <> pos2.brett[i,j] THEN exit;
      gleich := true
    END; {of gleich}

  FUNCTION vorhanden (pos: tPosition; VAR liste: tListe): boolean;
    {prft, ob Position in der Besuchtliste vorhanden ist}
    VAR
      i : integer;
    BEGIN
      vorhanden := false;
      IF liste.laenge > 0 THEN BEGIN
        FOR i := 1 TO liste.laenge DO
          IF gleich(liste.inhalt[i], pos) THEN BEGIN
            vorhanden := true; Ausgabe(pos, 1, 21);
            gotoXY(1, 23); write('Schon besucht!');
            write(chr(7)); readln; exit
          END {of then}
      END {of then}
    END; {of vorhanden}

  PROCEDURE zeige (VAR liste: tListe);
    {zeigt gesamte Liste von Spielpositionen}
    VAR
      i, zeile, spalte : integer;
    BEGIN
      clrScr; spalte := 1; zeile := 1;
      write('Aktueller Suchpfad (Tiefe ', liste.laenge, '): ');

      IF liste.laenge <= 0 THEN
        write('Die Liste ist leer.')
      ELSE BEGIN
        inc(zeile, 2);
        FOR i := 1 TO liste.laenge DO BEGIN
          Ausgabe(liste.inhalt[i], spalte, zeile);
          inc(spalte, 8);
          IF i MOD 10 = 0 THEN BEGIN
            spalte := 1; inc(zeile, 3)
          END {of then}
        END {of for}
      END {of then}
    END; {of zeige}

  PROCEDURE suche_Weg (position: tPosition; tiefe: integer);
    {rekursiver Tiefendurchlauf des Positionsgraphen}
    VAR
      zug        : tZug;
      nachfolger : tPosition;
    BEGIN
      fuege_an(position, tiefe, besuchtliste);

      zeige(besuchtliste); readln;

      IF tiefe < tiefenschranke THEN
        FOR zug := west TO sued DO
          IF zulaessig(zug, position) THEN BEGIN
            erzeuge(nachfolger, zug, position);
            IF NOT vorhanden(nachfolger, besuchtliste) THEN
              suche_Weg(nachfolger, tiefe + 1)
            END {of then}
    END; {of suche Weg}

  BEGIN {Hauptprogramm}
    Startangaben(anfangsposition);
    suche_Weg(anfangsposition, 1);
    writeln; writeln('Graphdurchlauf beendet.'); readln
  END.
